

/* mystring.h */


/*------------------------------------------------------------------------
 * my-library with string functions for the tPSZ type
 *------------------------------------------------------------------------
 */


/*
 * Before the first use of any string with one of the ppz-functions, the
 * string pointer pzStr has to be initialized with NULL. NULL is valid for
 * any pzStr and counts as an empty string. If a function would return an
 * empty string, it returns NULL, an empty string is never allocated by these
 * functions.
 * Any ppzStr is not allowed to be NULL. This is not checked, because NULL is
 * just one special case out of all possible wrong values.
 * The destination variable can be identical with any source variable in any
 * function. ppzStringConcatN(&pzName,2,pzName,pzName) is always possible.
 * pzName before: "Test"
 * pzName after : "TestTest"
 * The operations are done with the source strings; the result is stored in
 * the destination string without change of the source strings.
 *
 * There is no Create function, just a Reserve. A string is 'created' by
 * its initialization with NULL. Consequently, there is no Destroy, just
 * a Clear which frees the allocated memory and resets the string to NULL.
 * The string is still valid, not 'destroyed'.
 */


/*----------------------------------------------------------------------*/

#ifndef MYSTRING_H
#define MYSTRING_H

/*----------------------------------------------------------------------*/

#include "mytypes.h"

/*----------------------------------------------------------------------*/


tCOUNT cnStringLength    (tPSZc pzStr);

tPSZ   pzStringSubstChar (tPSZ pzStr, tCHAR cOld, tCHAR cNew);

/*---------------------------------------------*/

tpPSZ ppzStringReserve   (tpPSZ ppzStr, tCOUNT cnLength);
tpPSZ ppzStringClear     (tpPSZ ppzStr);
tpPSZ ppzStringMemUsage  (tpPSZ ppzStr, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tpPSZ ppzStringCopy      (tpPSZ ppzStr, tPSZc pzSrc);
tpPSZ ppzStringConcatN   (tpPSZ ppzStr, tCOUNT cnNumber, ...);
tpPSZ ppzStringMiddle    (tpPSZ ppzStr, tPSZc pzSrc, tCOUNT cnIndex, tCOUNT cnNumber);
tpPSZ ppzStringMultiChar (tpPSZ ppzStr, tCHAR cChar, tCOUNT cnNumber);

/*---------------------------------------------*/

#define uwStringNumMaxBase 16

tCOUNT cnStringNumStrSize    (tCOUNT cnWords, tUWORD uwBase);
tPSZ   pzStringNumIntToStr   (tPSZ pzStr, tpcUWORD puwInt, tCOUNT cnWords, tUWORD uwBase);

tpPSZ  ppzStringUIntToStr    (tpPSZ ppzStr, tpcUWORD puwInt, tCOUNT cnWords, tUWORD uwBase);
tpPSZ  ppzStringULongToStr   (tpPSZ ppzStr, tULONG ulInt, tUWORD uwBase);
tpPSZ  ppzStringSLongToStr   (tpPSZ ppzStr, tSLONG slInt, tUWORD uwBase);

tpPSZ  ppzStringFormatNumber (tpPSZ ppzStr, tPSZc pzNumber, tCOUNT cnZeroFill, tCOUNT cnBlkSize,
                              tPSZc pzBlkSep, tPSZc pzPrefix, tPSZc pzSuffix);


/*----------------------------------------------------------------------*/

#endif
