

/* statstne.c */


/*------------------------------------------------------------------------
 * Solitaire field statistics: Stones
 *------------------------------------------------------------------------
 * This version of statstne is part of the Solitaire Brute Force tool.
 * Copyright (C) 2001 Holger Hasselbach <H.Hasselbach@galad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------*/

#include "statstne.h"
#include "mymemory.h"
#include "statsymm.h"

#include <assert.h>

/*----------------------------------------------------------------------*/


struct s_STATSTNE
{
    tpsSTATSYMM apsSymms[cnFieldSymIndexRange];

    tpsSTATSYMM psSymmSum;
};

typedef struct s_STATSTNE tsSTATSTNE;


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Create a new STATSTNE
 *-----------------------------------------------
 */

tpsSTATSTNE psStatStneCreate (tVOID)
{
    tpsSTATSTNE psStne;
    tCOUNT      cnI;

    psStne = pvMemoryAllocObject(sSTATSTNE);

    for (cnI = 0; cnI < cnFieldSymIndexRange; cnI++)
        psStne->apsSymms[cnI] = psStatSymmCreate();

    psStne->psSymmSum = psStatSymmCreate();

    return (psStne);
}


/*-----------------------------------------------
 * Destroy a STATSTNE
 *-----------------------------------------------
 */

tpsSTATSTNE psStatStneDestroy (tpsSTATSTNE psStne)
{
    tCOUNT cnI;

    if (psStne)
      {
        for (cnI = 0; cnI < cnFieldSymIndexRange; cnI++)
            psStatSymmDestroy(psStne->apsSymms[cnI]);

        psStatSymmDestroy(psStne->psSymmSum);

        pvMemoryFree(psStne);
      }

    return (NULL);
}


/*-----------------------------------------------
 * Memory statistics
 *-----------------------------------------------
 */

tpsSTATSTNE psStatStneMemUsage (tpsSTATSTNE psStne, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks)
{
    tCOUNT cnI;

    if (psStne)
      {
        for (cnI = 0; cnI < cnFieldSymIndexRange; cnI++)
            psStatSymmMemUsage(psStne->apsSymms[cnI],pcnSizeof,pcnBlocks);

        psStatSymmMemUsage(psStne->psSymmSum,pcnSizeof,pcnBlocks);

        *pcnSizeof += cnMemorySizeofObject(sSTATSTNE);
        *pcnBlocks += 1;
      }

    return (psStne);
}


/*-----------------------------------------------
 * Add a field statistic
 *-----------------------------------------------
 */

tVOID vStatStneAddField (tpsSTATSTNE psStne, tpsFIELDSTT psStt)
{
    tCOUNT cnSymIndex;

    cnSymIndex = cnFieldSttGetSymIndex(psStt);
    assert(cnSymIndex < cnFieldSymIndexRange);

    vStatSymmAddField(psStne->apsSymms[cnSymIndex],psStt);
}


/*-----------------------------------------------
 * Add a STATSTNE to another STATSTNE
 *-----------------------------------------------
 */

tVOID vStatStneAddObject (tpsSTATSTNE psStne, tpsSTATSTNE psAdd)
{
    tCOUNT cnI;

    for (cnI = 0; cnI < cnFieldSymIndexRange; cnI++)
        vStatSymmAddObject(psStne->apsSymms[cnI],psAdd->apsSymms[cnI]);

    vStatSymmAddObject(psStne->psSymmSum,psAdd->psSymmSum);
}


/*-----------------------------------------------
 * Calculate statistic
 *-----------------------------------------------
 */

tVOID vStatStneCalcWhole (tpsSTATSTNE psStne)
{
    tCOUNT       cnI;
    tppsSTATSYMM ppsSymm;

    ppsSymm = psStne->apsSymms;
    for (cnI = 0; cnI < cnFieldSymIndexRange; cnI++, ppsSymm++)
      {
        vStatSymmCalcWhole(*ppsSymm);
        vStatSymmAddObject(psStne->psSymmSum,*ppsSymm);
      }
}


/*-----------------------------------------------
 * Test if there is any pattern at all
 *-----------------------------------------------
 */

tBOOL bStatStneIsEmpty (tpsSTATSTNE psStne)
{
    return ( bStatSymmIsEmpty(psStne->psSymmSum) );
}


/*-----------------------------------------------
 * Get the maximum number of moves
 *-----------------------------------------------
 */

tCOUNT cnStatStneMaxMoves (tpsSTATSTNE psStne)
{
    return ( cnStatSymmMaxMoves(psStne->psSymmSum) );
}


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Store parameters into table
 *-----------------------------------------------
 * NULL is not valid for pzContent
 * Content qualifier:
 * "S" : Symms[cnSymm]
 * "Sn": Symms[n] with n = decimal number
 *       SymmSum if cnSymm or n >= SymIndexRange
 * "+" : SymmSum
 *-----------------------------------------------
 */

tVOID vStatStneTabSet (tpsSTATSTNE psStne, tpsSTATTABS psTabs, tPSZc pzContent)
{
    tCOUNT cnSymm;

    switch (*pzContent)
      {
      case 'S' :
        cnSymm    = cnStatTabsGetSymm(psTabs);
        pzContent = pzStatTabsStrGetNumber(pzContent + 1,&cnSymm);
        if (cnSymm < cnFieldSymIndexRange)
          {
            vStatTabsStyleEntryEnum(psTabs);
            vStatSymmTabSet(psStne->apsSymms[cnSymm],psTabs,pzContent);
          }
        else
          {
            vStatTabsStyleEntryAdd(psTabs);
            vStatSymmTabSet(psStne->psSymmSum,psTabs,pzContent);
          }
        break;

      case '+' :
        vStatTabsStyleEntryAdd(psTabs);
        vStatSymmTabSet(psStne->psSymmSum,psTabs,pzContent + 1);
        break;

      default : assert(FALSE);
      }
}


/*----------------------------------------------------------------------*/
